/*
 * parseconfig.c - Functions for parsing a configuration file
 *
 * Copyright (C) 2001  ViaLink Internet S/C Ltda, Rio de Janeiro, Brazil
 *                     http://www.vialink.com.br/
 * Author:             Juan Carlos Castro y Castro <jcastro@vialink.com.br>
 *
 * See the README file for more info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, you can either send email to this
 * program's maintainer or write to: The Free Software Foundation,
 * Inc.; 59 Temple Place, Suite 330; Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>

#include "parseconfig.h"

void parseconfig(char *configfile, PCFGOPTION *options)
{
	char		*pstart, *pend, *oldstring, *newstring;
	FILE		*streami = fopen(configfile, "r");
	PCFGOPTION	*poption, *optionsvector = options;
	int		lenarg;
	char		c;
	static char	buf[514];

	if (!streami)
		return;

	while (fgets(buf, sizeof(buf), streami))
	{
		/* Weed off CRs, LFs, and blanks from the end */
		removecrlf(buf);
		removespace(buf);

		/* Here, I should warn about excessively long lines */
		/* -- I'll do that when I implement logging         */

		/* Locate first nonspace */
		pstart = gotononspace(buf);

		/* Skip commented-out and blank lines */
		c = *pstart;
		if ((c == '#') || (c == ';') || (c = 0))
			continue;

		/* Locate next blankspace (end of token) */
		pend = gotospace(pstart);

		/* Locate next nonspace (start of argument) */
		if (*pend)
		{
			*pend++ = 0;
			pend = gotononspace(pend);
		}

		/* Un-quote argument */
		lenarg = strlen(pend);
		if (lenarg > 1)
		{
			if (*pend == '\"')
			{
				if (pend[lenarg - 1] == '\"')
				{
					pend[lenarg - 1] = 0;
					pend++;
				}
			}
			else if (*pend == '\'')
			{
				if (pend[lenarg - 1] == '\'')
				{
					pend[lenarg - 1] = 0;
					pend++;
				}
			}
		}

		/* See if it's a valid token */
		poption = optionsvector;
		while (poption->name)
		{
			if (!strcmp(pstart, poption->name))
			{
				switch(poption->type)
				{
					case PCFG_INT:
					*((int *) poption->value)
					      = atoi(pend);
					break;

					case PCFG_UINT:
					*((unsigned int *) poption->value)
					      = strtoul(pend, NULL, 10);
					break;

					case PCFG_STRING:
					oldstring = *((char **) poption->value);
					newstring = malloc(poption->size);
					if (!newstring)
						break;
					if (oldstring)
						free(oldstring);
					*((char **) poption->value) = newstring;
					strncpy(newstring, pend, poption->size);
					break;

					case PCFG_BOOLEAN:
					if (!strcmp(pend, "no"))
						*((int *) poption->value) = 0;
					else if (!strcmp(pend, "yes"))
						*((int *) poption->value) = 1;
					else
						*((int *) poption->value)
						      = (atoi(pend) ? 1 : 0);
					break;

					case PCFG_JUMPSTATE:
					optionsvector =
					        (PCFGOPTION *) poption->value;
					if (poption->postprocess)
						poption->postprocess();
				}
			}
			poption++;
		}
	}
	fclose(streami);
}

/* Find next whitespace character (stop at end of string) */

char *gotospace(char *str)
{
	char	c;

	for (;;)
	{
		c = *str;
		if ((c == ' ') || (c == '\t') || (c == 0))
			return str;
		str++;
	}
}

/* Find next nonspace character (stop at end of string) */

char *gotononspace(char *str)
{
	char	c;

	for (;;)
	{
		c = *str;
		if ((c != ' ') && (c != '\t'))
			return str;
		str++;
	}
}

void removecrlf(char *str)
{
	char	*pend;

	if (!str[0])
		return;

	pend = &str[strlen(str) - 1];
	while (((*pend == '\r') || (*pend == '\n')) && (pend >= str))
	{
		*pend-- = 0;
	}
}

void removespace(char *str)
{
	char	*pend;

	if (!str[0])
		return;

	pend = &str[strlen(str) - 1];
	while (((*pend == ' ') || (*pend == '\t')) && (pend >= str))
	{
		*pend-- = 0;
	}
}
